/*
 * Nightshade (TM) astronomy simulation and visualization
 *
 * Author: Trystan A. Larey-Williams
 * Copyright (C) 2010 Digitalis Education Solutions, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Nightshade is a trademark of Digitalis Education Solutions, Inc.
 * See the TRADEMARKS file for trademark usage requirements.
 *
 */

#include "nshade_shared_memory_connection.h"

using namespace boost::interprocess;
using namespace std;

NshadeSharedMemoryConnection::NshadeSharedMemoryConnection( managed_shared_memory* const shm ) : m_dirty(false) {
	m_mutex = shm->find<interprocess_mutex>(unique_instance).first;
	m_cmdBuf = shm->find_or_construct<NshadeCommand>(unique_instance)();
}

NshadeSharedMemoryConnection::~NshadeSharedMemoryConnection() {

}

void NshadeSharedMemoryConnection::SetDirty() {
	m_dirty = true;
}

void NshadeSharedMemoryConnection::SetClientType( NshadeSharedMemoryConnection::Type type ) {
	m_clientType = type;
}

NshadeSharedMemoryConnection::Type NshadeSharedMemoryConnection::ClientType( void ) {
	return m_clientType;
}

void NshadeSharedMemoryConnection::SetOwner( const unsigned long owner ) {
	m_owner = owner;
}

const unsigned long NshadeSharedMemoryConnection::Owner( void ) {
	return m_owner;
}

void NshadeSharedMemoryConnection::Write( std::string data ) {
	m_mutex->lock();
	try {
		unsigned int len = data.length();
		if( len >= m_cmdBuf->szBuf )
			len = m_cmdBuf->szBuf - 1;

		memcpy( (void*)m_cmdBuf->buf, (void*)data.c_str(), len );
		m_cmdBuf->length = len;
		m_cmdBuf->sender = this;
	}
	catch( ... ) {
		m_mutex->unlock();
		throw;
	}
	m_mutex->unlock();
}

void NshadeSharedMemoryConnection::WriteResponse( std::string data ) {
	m_mutex->lock();
	try {
		unsigned int len = data.length();
		if( len >= m_cmdBuf->szResponse )
			len = m_cmdBuf->szResponse - 1;

		memcpy( (void*)m_cmdBuf->response, (void*)data.c_str(), len );
		m_cmdBuf->responseLen = len;
		if( m_cmdBuf->sender.get_offset() > 1 )
			m_cmdBuf->sender->m_dirty = true;
	}
	catch( ... ) {
		m_mutex->unlock();
		throw;
	}
	m_mutex->unlock();
}

bool NshadeSharedMemoryConnection::Read( std::string& data ) {
	bool have_data = false;

	m_mutex->lock();
	try {
		if( m_dirty ) {
			data = std::string( m_cmdBuf->buf, m_cmdBuf->length );
			have_data = true;
		}
	}
	catch( ... ) {
		m_mutex->unlock();
		throw;
	}
	m_dirty = false;
	m_mutex->unlock();
	return have_data;
}

bool NshadeSharedMemoryConnection::ReadResponse( std::string& data ) {
	bool have_data = false;

	m_mutex->lock();
	try {
		if( m_dirty ) {
			data = std::string( m_cmdBuf->response, m_cmdBuf->responseLen );
			have_data = true;
		}
	}
	catch( ... ) {
		m_mutex->unlock();
		throw;
	}
	m_dirty = false;
	m_mutex->unlock();
	return have_data;
}
