/*
 * Nightshade (TM) astronomy simulation and visualization
 *
 * Author: Trystan A. Larey-Williams
 * Copyright (C) 2010 Digitalis Education Solutions, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Nightshade is a trademark of Digitalis Education Solutions, Inc.
 * See the TRADEMARKS file for trademark usage requirements.
 *
 */

#ifndef SHAREDMEMORYCONNECTION_H_
#define SHAREDMEMORYCONNECTION_H_

#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/interprocess/sync/interprocess_mutex.hpp>
#include <string>
#include <iostream>

class NshadeSharedMemoryConnection;

struct NshadeCommand {
	NshadeCommand(){};
	unsigned int length;
	unsigned int responseLen;
	boost::interprocess::offset_ptr<NshadeSharedMemoryConnection> sender;
	static const unsigned short szBuf = 4096;
	static const unsigned short szResponse = 1024;
	char buf[szBuf];
	char response[szResponse];
};

class NshadeSharedMemoryConnection {
public:
	NshadeSharedMemoryConnection( boost::interprocess::managed_shared_memory* );
	virtual ~NshadeSharedMemoryConnection();

	enum Type {
		nightshade,
		fcgi,
		user
	};

	bool Read( std::string& data );
	void Write( std::string data );
	bool ReadResponse( std::string& data );
	void WriteResponse( std::string data );
	void SetDirty( void );
	void SetOwner( const unsigned long );
	const unsigned long Owner( void );
	void SetClientType( Type );
	Type ClientType( void );

private:
	boost::interprocess::offset_ptr<boost::interprocess::interprocess_mutex> m_mutex;
	boost::interprocess::offset_ptr<NshadeCommand> m_cmdBuf;
	bool m_dirty;
	unsigned long m_owner;
	Type m_clientType;
};

#endif /* SHAREDMEMORYCONNECTION_H_ */
